@@ -13,6 +13,7 @@
 import datetime
 import itertools
 import mock
+from webtest.app import AppError
 
 from oslo_config import cfg
 from oslo_serialization import jsonutils
@@ -36,6 +37,7 @@ def post_get_test_audit_template(**kw):
     strategy = db_utils.get_test_strategy(goal_id=goal['id'])
     kw['goal'] = kw.get('goal', goal['uuid'])
     kw['strategy'] = kw.get('strategy', strategy['uuid'])
+    kw['scope'] = kw.get('scope', [])
     audit_template = api_utils.audit_template_post_data(**kw)
     return audit_template
 
@@ -229,8 +231,7 @@ class TestListAuditTemplate(FunctionalTestWithSetup):
         self.assertIn(next_marker, response['next'])
 
     def test_collection_links_default_limit(self):
-        cfg.CONF.set_override('max_limit', 3, 'api',
-                              enforce_type=True)
+        cfg.CONF.set_override('max_limit', 3, 'api')
         for id_ in range(5):
             obj_utils.create_test_audit_template(
                 self.context, id=id_, uuid=utils.generate_uuid(),
@@ -293,6 +294,50 @@ class TestListAuditTemplate(FunctionalTestWithSetup):
             '/audit_templates?strategy=%s' % self.fake_strategy2.name)
         self.assertEqual(2, len(response['audit_templates']))
 
+    def test_many_with_sort_key_name(self):
+        audit_template_list = []
+        for id_ in range(1, 6):
+            audit_template = obj_utils.create_test_audit_template(
+                self.context, id=id_, uuid=utils.generate_uuid(),
+                name='My Audit Template {0}'.format(id_))
+            audit_template_list.append(audit_template)
+
+        response = self.get_json('/audit_templates?sort_key=%s' % 'name')
+
+        names = [s['name'] for s in response['audit_templates']]
+
+        self.assertEqual(
+            sorted([at.name for at in audit_template_list]),
+            names)
+
+    def test_many_with_sort_key_goal_name(self):
+        goal_names_list = []
+        for id_, goal_id in enumerate(itertools.chain.from_iterable([
+                itertools.repeat(self.fake_goal1.id, 3),
+                itertools.repeat(self.fake_goal2.id, 2)]), 1):
+            audit_template = obj_utils.create_test_audit_template(
+                self.context, id=id_, uuid=utils.generate_uuid(),
+                name='My Audit Template {0}'.format(id_),
+                goal_id=goal_id)
+            goal_names_list.append(audit_template.goal.name)
+
+        for direction in ['asc', 'desc']:
+            response = self.get_json(
+                '/audit_templates?sort_key={0}&sort_dir={1}'
+                .format('goal_name', direction))
+
+            goal_names = [s['goal_name'] for s in response['audit_templates']]
+
+            self.assertEqual(
+                sorted(goal_names_list, reverse=(direction == 'desc')),
+                goal_names)
+
+    def test_sort_key_validation(self):
+        response = self.get_json(
+            '/audit_templates?sort_key=%s' % 'goal_bad_name',
+            expect_errors=True)
+        self.assertEqual(400, response.status_int)
+
 
 class TestPatch(FunctionalTestWithSetup):
 
@@ -510,6 +555,66 @@ class TestPost(FunctionalTestWithSetup):
             response.json['created_at']).replace(tzinfo=None)
         self.assertEqual(test_time, return_created_at)
 
+    @mock.patch.object(timeutils, 'utcnow')
+    def test_create_audit_template_with_strategy_name(self, mock_utcnow):
+        audit_template_dict = post_get_test_audit_template(
+            goal=self.fake_goal1.uuid,
+            strategy=self.fake_strategy1.name)
+        test_time = datetime.datetime(2000, 1, 1, 0, 0)
+        mock_utcnow.return_value = test_time
+
+        response = self.post_json('/audit_templates', audit_template_dict)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(201, response.status_int)
+        # Check location header
+        self.assertIsNotNone(response.location)
+        expected_location = \
+            '/v1/audit_templates/%s' % response.json['uuid']
+        self.assertEqual(urlparse.urlparse(response.location).path,
+                         expected_location)
+        self.assertTrue(utils.is_uuid_like(response.json['uuid']))
+        self.assertNotIn('updated_at', response.json.keys)
+        self.assertNotIn('deleted_at', response.json.keys)
+        self.assertEqual(self.fake_goal1.uuid, response.json['goal_uuid'])
+        self.assertEqual(self.fake_strategy1.uuid,
+                         response.json['strategy_uuid'])
+        self.assertEqual(self.fake_strategy1.name,
+                         response.json['strategy_name'])
+        return_created_at = timeutils.parse_isotime(
+            response.json['created_at']).replace(tzinfo=None)
+        self.assertEqual(test_time, return_created_at)
+
+    def test_create_audit_template_validation_with_aggregates(self):
+        scope = [{'compute': [{'host_aggregates': [{'id': '*'}]},
+                              {'availability_zones': [{'name': 'AZ1'},
+                                                      {'name': 'AZ2'}]},
+                              {'exclude': [
+                                  {'instances': [
+                                      {'uuid': 'INSTANCE_1'},
+                                      {'uuid': 'INSTANCE_2'}]},
+                                  {'compute_nodes': [
+                                      {'name': 'Node_1'},
+                                      {'name': 'Node_2'}]},
+                                  {'host_aggregates': [{'id': '*'}]}
+                              ]}
+                              ]
+                  }
+                 ]
+        audit_template_dict = post_get_test_audit_template(
+            goal=self.fake_goal1.uuid,
+            strategy=self.fake_strategy1.uuid, scope=scope)
+        with self.assertRaisesRegex(AppError,
+                                    "be included and excluded together"):
+            self.post_json('/audit_templates', audit_template_dict)
+
+        scope = [{'host_aggregates': [{'id1': '*'}]}]
+        audit_template_dict = post_get_test_audit_template(
+            goal=self.fake_goal1.uuid,
+            strategy=self.fake_strategy1.uuid, scope=scope)
+        response = self.post_json('/audit_templates',
+                                  audit_template_dict, expect_errors=True)
+        self.assertEqual(500, response.status_int)
+
     def test_create_audit_template_does_autogenerate_id(self):
         audit_template_dict = post_get_test_audit_template(
             goal=self.fake_goal1.uuid, strategy=None)
@@ -587,6 +692,26 @@ class TestPost(FunctionalTestWithSetup):
         self.assertEqual(400, response.status_int)
         assert not cn_mock.called
 
+    def test_create_audit_template_with_old_scope(self):
+        scope = [{'host_aggregates': [{'id': '*'}]},
+                 {'availability_zones': [{'name': 'AZ1'},
+                                         {'name': 'AZ2'}]},
+                 {'exclude': [
+                     {'instances': [
+                         {'uuid': 'INSTANCE_1'},
+                         {'uuid': 'INSTANCE_2'}]},
+                     {'compute_nodes': [
+                         {'name': 'Node_1'},
+                         {'name': 'Node_2'}]},
+                 ]}
+                 ]
+        audit_template_dict = post_get_test_audit_template(
+            goal=self.fake_goal1.uuid,
+            strategy=self.fake_strategy1.uuid, scope=scope)
+        response = self.post_json('/audit_templates',
+                                  audit_template_dict)
+        self.assertEqual(201, response.status_int)
+
 
 class TestDelete(api_base.FunctionalTest):
 
